@@ -1,4 +1,7 @@
 //! Interpreter
+use bounded_vec::BoundedVecOutOfBounds;
+use sigma_ser::ScorexParsingError;
+use sigma_ser::ScorexSerializationError;
 use std::rc::Rc;
 
 use ergotree_ir::ergo_tree::ErgoTreeError;
@@ -24,6 +27,7 @@ pub mod env;
 
 pub(crate) mod and;
 pub(crate) mod apply;
+pub(crate) mod atleast;
 pub(crate) mod bin_op;
 pub(crate) mod bit_inversion;
 pub(crate) mod block;
@@ -44,6 +48,7 @@ pub(crate) mod coll_slice;
 pub(crate) mod collection;
 pub(crate) mod cost_accum;
 pub(crate) mod costs;
+pub(crate) mod create_avl_tree;
 pub(crate) mod create_prove_dh_tuple;
 pub(crate) mod create_provedlog;
 pub(crate) mod decode_point;
@@ -78,6 +83,7 @@ pub(crate) mod sbox;
 pub(crate) mod scoll;
 pub(crate) mod scontext;
 pub(crate) mod select_field;
+pub(crate) mod sglobal;
 pub(crate) mod sgroup_elem;
 pub(crate) mod sheader;
 pub(crate) mod sigma_and;
@@ -86,6 +92,7 @@ pub(crate) mod sigma_prop_bytes;
 pub(crate) mod soption;
 pub(crate) mod spreheader;
 pub(crate) mod subst_const;
+pub(crate) mod tree_lookup;
 pub(crate) mod tuple;
 pub(crate) mod upcast;
 pub(crate) mod val_use;
@@ -137,9 +144,19 @@ pub enum EvalError {
     /// Not yet implemented
     #[error("evaluation is not yet implemented: {0}")]
     NotImplementedYet(&'static str),
+    /// Invalid item quantity for BoundedVec
+    #[error("Invalid item quantity for BoundedVec: {0}")]
+    BoundedVecError(#[from] BoundedVecOutOfBounds),
+    /// Scorex serialization error
+    #[error("Serialization error: {0}")]
+    ScorexSerializationError(#[from] ScorexSerializationError),
+    /// Scorex serialization parsing error
+    #[error("Serialization parsing error: {0}")]
+    ScorexParsingError(#[from] ScorexParsingError),
 }
 
 /// Result of expression reduction procedure (see `reduce_to_crypto`).
+#[derive(PartialEq, Debug, Clone)]
 pub struct ReductionResult {
     /// value of SigmaProp type which represents a statement verifiable via sigma protocol.
     pub sigma_prop: SigmaBoolean,
@@ -147,32 +164,28 @@ pub struct ReductionResult {
     pub cost: u64,
 }
 
-/// Interpreter
-pub trait Evaluator {
-    /// Evaluate the given expression by reducing it to SigmaBoolean value.
-    fn reduce_to_crypto(
-        &self,
-        expr: &Expr,
-        env: &Env,
-        ctx: Rc<Context>,
-    ) -> Result<ReductionResult, EvalError> {
-        let cost_accum = CostAccumulator::new(0, None);
-        let mut ectx = EvalContext::new(ctx, cost_accum);
-        expr.eval(env, &mut ectx)
-            .and_then(|v| -> Result<ReductionResult, EvalError> {
-                match v {
-                    Value::Boolean(b) => Ok(ReductionResult {
-                        sigma_prop: SigmaBoolean::TrivialProp(b),
-                        cost: 0,
-                    }),
-                    Value::SigmaProp(sp) => Ok(ReductionResult {
-                        sigma_prop: sp.value().clone(),
-                        cost: 0,
-                    }),
-                    _ => Err(EvalError::InvalidResultType),
-                }
-            })
-    }
+/// Evaluate the given expression by reducing it to SigmaBoolean value.
+pub fn reduce_to_crypto(
+    expr: &Expr,
+    env: &Env,
+    ctx: Rc<Context>,
+) -> Result<ReductionResult, EvalError> {
+    let cost_accum = CostAccumulator::new(0, None);
+    let mut ectx = EvalContext::new(ctx, cost_accum);
+    expr.eval(env, &mut ectx)
+        .and_then(|v| -> Result<ReductionResult, EvalError> {
+            match v {
+                Value::Boolean(b) => Ok(ReductionResult {
+                    sigma_prop: SigmaBoolean::TrivialProp(b),
+                    cost: 0,
+                }),
+                Value::SigmaProp(sp) => Ok(ReductionResult {
+                    sigma_prop: sp.value().clone(),
+                    cost: 0,
+                }),
+                _ => Err(EvalError::InvalidResultType),
+            }
+        })
 }
 
 #[derive(Debug)]
@@ -209,7 +222,12 @@ fn smethod_eval_fn(method: &SMethod) -> Result<EvalFn, EvalError> {
             savltree::IS_UPDATE_ALLOWED_METHOD_ID => self::savltree::IS_UPDATE_ALLOWED_EVAL_FN,
             savltree::IS_REMOVE_ALLOWED_METHOD_ID => self::savltree::IS_REMOVE_ALLOWED_EVAL_FN,
             savltree::UPDATE_OPERATIONS_METHOD_ID => self::savltree::UPDATE_OPERATIONS_EVAL_FN,
+            savltree::GET_METHOD_ID => self::savltree::GET_EVAL_FN,
+            savltree::GET_MANY_METHOD_ID => self::savltree::GET_MANY_EVAL_FN,
             savltree::INSERT_METHOD_ID => self::savltree::INSERT_EVAL_FN,
+            savltree::CONTAINS_METHOD_ID => self::savltree::CONTAINS_EVAL_FN,
+            savltree::REMOVE_METHOD_ID => self::savltree::REMOVE_EVAL_FN,
+            savltree::UPDATE_METHOD_ID => self::savltree::UPDATE_EVAL_FN,
             method_id => {
                 return Err(EvalError::NotFound(format!(
                     "Eval fn: unknown method id in SAvlTree: {:?}",
@@ -222,6 +240,9 @@ fn smethod_eval_fn(method: &SMethod) -> Result<EvalFn, EvalError> {
             scontext::SELF_BOX_INDEX_PROPERTY_METHOD_ID => self::scontext::SELF_BOX_INDEX_EVAL_FN,
             scontext::HEADERS_PROPERTY_METHOD_ID => self::scontext::HEADERS_EVAL_FN,
             scontext::PRE_HEADER_PROPERTY_METHOD_ID => self::scontext::PRE_HEADER_EVAL_FN,
+            scontext::LAST_BLOCK_UTXO_ROOT_HASH_PROPERTY_METHOD_ID => {
+                self::scontext::LAST_BLOCK_UTXO_ROOT_HASH_EVAL_FN
+            }
             method_id => {
                 return Err(EvalError::NotFound(format!(
                     "Eval fn: unknown method id in SContext: {:?}",
@@ -315,6 +336,17 @@ fn smethod_eval_fn(method: &SMethod) -> Result<EvalFn, EvalError> {
                 )))
             }
         },
+        sglobal::TYPE_CODE => match method.method_id() {
+            sglobal::GROUP_GENERATOR_METHOD_ID => self::sglobal::GROUP_GENERATOR_EVAL_FN,
+            sglobal::XOR_METHOD_ID => self::sglobal::XOR_EVAL_FN,
+            method_id => {
+                return Err(EvalError::NotFound(format!(
+                    "Eval fn: method {:?} with method id {:?} not found in SGlobal",
+                    method.name(),
+                    method_id,
+                )))
+            }
+        },
         type_id => {
             return Err(EvalError::NotFound(format!(
                 "Eval fn: unknown type id {:?}",
